<?php
/**
* Interface Parent Class
*
* The interface class contains all the methods required to handle user 
* input from the web interface and handles any specialized handling of the
* display for these requests. Child classes often override the stub methods
* for data handling and internal block parsing.
*
* @version  0.2.0 beta
* @author   Fred Hirsch
* @copyright  Dark Hart Design  2001
* @package CORE
*/
class Interface {
  var $interface;
  var $interface_data = array();
  var $theme;
  var $session;
  var $command;

/**
* Interface Constructor
*
* The class constructor requires the input of an interface, operation and
* plugin name. Without these elements, the system will only build the default
* interface which contains no inherent functionality. These core elements are
* used during initialization to create the proper template and theme elements.
*
* @param 	global String 	$interface		The interface name or description which is cross referenced to blocks in the database.
* @param  global String		$operation		The current operation being performed (blank by default).
* @param 	global String		$plugin			The name of the current plugin being used for DB cross references.
* @param 	global String		$interface_title		A title most often used to name the page in the browser title.
* @return VOID
*/
  function Interface() {
    global $interface;
    global $operation;
    global $plugin;
    global $interface_title;

    $this->interface_title = $interface_title;
    $this->interface = $interface;
    $this->operation = $operation;
    $this->plugin = $plugin;
    unset($operation); unset($interface);unset ($plugin);
  }

/**
* Class Initializer
*
* The initializer performs a number of key tasks that are crucial to the 
* setup of the interface. These methods are set here due to an issue in PHP
* in which the object cannot cross-reference itself in other object constructors
* during instantiation. Each child interface may define its own initializeLocal
* method, but should never override the parent method.
*
* @see initializeLocal
*/
  function Initialize() {
    global $config;
    $this->command = new PluginMethodFactory();
    if (!$this->search[loaded]) {
      $this->search = $this->command->loadSearch();
    }
    $this->form_data = empty($this->form_data) ? array() : $this->form_data;
    $this->form_data = array_merge($this->form_data, $this->command->loadFormData());
    $this->command->loadUserPrefs();
    $this->command->loadConfig();

    // If our program is searching for data to display, we should retrieve it.
    if (isset($this->search[id]) && $this->search[id] != '')
      $this->interface_data = array_merge($this->interface_data, $this->command->getInterfaceData($this->search, $this->flags[transpose]));

    empty($this->theme) ? $this->theme = $this->resetTheme() : $this->theme->initTheme();
    //$this->theme = new Theme($this->themeID);
    $this->command->parseContent($this->theme, $this->interface_data);
    method_exists($this, 'initializeLocal') ? $this->initializeLocal() : '';

    if ($config[track_users] == true) {
      //$tracking_interface = (isset($this->operation)) ? "{$this->interface}.{$this->operation}" : "{$this->interface}";
      $tracking_interface = $this->interface;
      $this->tracking = $config[article][tracking][$tracking_interface];
      $this->command->trackUser($this);
    }
    
    // A theme is then instantiated for the Interface using either a system-wide
    // configuration variable, or a custom theme selected by the user. The title
    // of the theme/interface is currently kept in this area. This will be changed.
    if (empty($this->permission) || $this->permission == '') {
      $this->permission = 0;
    }

    $this->theme->tconfig[title] = $this->interface_title;
    
    //$this->theme->assign('config', $this->theme->tconfig);
    //$this->theme->assign('plugin', $this->plugin);
  }

  function Finalize () {
    global $userinfo, $config;
    $this->finalizeLocal();
    if ($userinfo[SAVE_CORE] || $userinfo[SAVE_PLUGIN]) {
        $this->command->saveUserPrefs();
        unset($userinfo[$this->plugin]);
    } else {
        return;
    }
  }
/**
* Reset Theme
*/
  function &resetTheme () {
    global $config;
    global $userinfo;
    if (isset($this->themeID) && $this->themeID != '') {
      $theme = $this->themeID;
    } elseif (isset($userinfo[theme]) && $userinfo[theme] != '') {
      $theme = $userinfo[theme];
    } else {
      $theme = $config[site_theme];
    }
    $this->themeID = $theme;
    $themeObj = new Theme($theme);
    $themeObj->tconfig[title] = $this->interface_title;
    $themeObj->assign('tconfig', $themeObj->tconfig);
    return $themeObj;
  }
  
/**
* Display Interface
* 
* The display method calls upon the theme object to perform parsing of the
* various areas of the template and return the completely parsed text from the
* template display method.
*/
  function &display() {
    $out = $this->theme->display();
    return $out;
  }

/**
* Main Stub Method
*
*/
  function main(&$theme) {    
    $theme->assign('idata', $this->interface_data);
  }
  function toc(&$theme) {
  }
/**
* Header Stub Method
*
*/
  function header(&$theme) {
  }
/**
* LeftBlock Stub Method
*
*/
  function leftblock(&$theme) {
  }
/**
* RightBlock Stub Method
*
*/
  function rightblock(&$theme) {
  }
/**
* Footer Stub Method
*
*/
  function footer(&$theme) {
  }

/**
* Search Stub Method
*
* The following is a sample base operations method. As you can see, it is very
* simple. Base operation methods allow the Interface to display forms or screens
* which contain data based on a selected operation. These differ from submission
* methods which must perform some task on the user's data. The base method only
* loads the interface data and sets the operation. The theme class will load 
* the operation blocks and parse the assigned data into the display.
*/
  function search() {
    $this->theme->assign('idata', $this->interface_data);
  }
/**
* Add Stub Method
*/
  function add() {
    $this->theme->assign('idata', $this->interface_data);
  }
/**
* Edit Stub Method
*/
  function edit() {
    $this->theme->assign('idata', $this->interface_data);
  }
/**
* Delete Stub Method
*/
  function delete() {
    $this->theme->assign('idata', $this->interface_data);
  }
/**
* Unknown Error
*
* A fallback method for displaying an error when a particular operation 
* fails because of odd reasons. The typical use of this method is to catch
* operation calls that are not implemented. This may happen if a user 
* attempts to submit a query on the interface using a method entered manually.
* This might otherwise be considered "hacking" on the Interface.
*/
  function unknownError() {
    $this->interface_data[error] = "The request submitted has caused an unknown error, please contact the developer of this product.";
    $this->theme->assign('idata', $this->interface_data);
  }

/**
* Validate HTML Form
*
*/
  function validateForm() {
	  global $dbh;
	  global $gui;
    global $config;
	  global $form__name;
	  global $form__operation;
	  $error = '';
	  $name_q = $this->command->db_quote($form__name);
	  $oper_q = $this->command->db_quote($form__operation);
	  $int_q  = $this->command->db_quote($this->interface);
	  $plug_q = $this->command->db_quote($this->plugin);
	  $sql = <<<END_SQL
  select name, type, title, is_required, is_unique, size_min, size_max, source_name from dyna_form_element
  where form_name = $name_q
  and operation in ('', $oper_q)
  and interface in ('', $int_q)
  and plugin in ('', $plug_q)
  order by ord
END_SQL;
  $result = $dbh->Execute($sql);
  !$result ? $error = "SQL: $sql -- " . $dbh->ErrorMsg() : '';
 	  
  //$result->MoveFirst();
//print "<h2>VVV $sql</h2>";exit;
  while ($data = $result->fields) {
	  if ($data[type] == 'checkbox') {
		  if (empty($this->form_data[$data[name]]) || $this->form_data[$data[name]] == '') { 
		    $this->form_data[$data[name]] = 0;
	    } 
    }
    if ($data[type] == 'file') {
      global $HTTP_POST_FILES;
      if (! is_dir("$config[site_path]$data[source_name]")) {
        mkdir("$config[site_path]$data[source_name]");
      }
      if ($this->form_data["$data[name]_remove"] == 1) {
        unlink("$config[site_path]{$this->interface_data[$data[name]]}")
          or $this->interface_data[error] .= 'Could not delete file.';
        unset($this->form_data["$data[name]_remove"]);
      } elseif (is_uploaded_file($HTTP_POST_FILES["form_$data[name]"][tmp_name]) && isset($this->form_data[id])) {
        preg_match("/\.([^\.]+)$/" , $HTTP_POST_FILES["form_$data[name]"][name], $ext);
        move_uploaded_file($HTTP_POST_FILES["form_$data[name]"][tmp_name], "$config[site_path]$data[source_name]/{$this->form_data[id]}.$ext[1]");
        $this->form_data[$data[name]] = "$data[source_name]/{$this->form_data[id]}.$ext[1]";
      } else {
        $this->form_data[$data[name]] = $gui->interface_data[$data[name]];
      } 
      unset($this->form_data[image_size], $this->form_data[image_name],  $this->form_data[image_type]);
    }
    
	  if (isset($this->form_data[$data[name]])) {
		  // Size checking of the fields
		  if ($data[is_required] == 1 && $this->form_data[$data[name]] == '') {
			  $error .= "$data[title] must have a value entered.\n";
		  }				  
		  if ($data[type] == 'textarea' || $data[type] == 'text') {
  		  if ($data[size_max] > 0 &&
  	      (strlen( $this->form_data[$data[name]] ) > $data[size_max] ) ) {
  			  $error .= "$data[title] cannot be more than $data[size_max] in length.\n";
  		  }
		  }
		  if ($data[size_min] > 0 &&
			  ($this->form_data[$data[name]] != '') &&
			  (strlen($this->form_data[$data[name]]) < $data[size_min])) {
			  $error .= "$data[title] must be at least $data[size_min] characters long.\n";
		  }
      if ($data[is_unique] == 1 && 
      		preg_match("/add|copy/", $gui->operation)) {
        $id_q = $dbh->qstr($this->form_data[$data[name]]);
        $sql = "select $data[name] from $data[source_name] where $data[name] = $id_q";
        $resunq = $dbh->Execute($sql);
        !$resunq ? $error = "SQL: $sql -- " . $dbh->ErrorMsg() : '';
        if ($resunq->fields[$data[name]] != '') {
          $error .= "$data[title] must be a unique value. The name <i>{$this->form_data[$data[name]]}</i> has already been used by a previous user.\n";
        }
      }
		  if ($data[type] == 'number') {
			  $testint = (int) $this->form_data[$data[name]];
			  if (!$testint){
				  $error .= "$data[title] is not an number value.\n";
			  } 
		  } elseif ($data[type] == 'password' || $data[type] == 'alphanum') {
			  if (preg_match_all("/\W/", $this->form_data[$data[name]], $matches)) {
				  $string = join(',', $matches[0]);
				  $error .= "$data[title] can only contain alphanumeric characters (Invalid: $string)\n";
			  }      
		  }	else { # Type is text
		  }
  	}
	  if ($data[type] == 'date' || $data[type] == 'datetime') {
		  if (isset($this->form_data["$data[name]_month"]) &&
	  		isset($this->form_data["$data[name]_day"]) &&
	  		isset($this->form_data["$data[name]_year"])) {
	  	  $this->form_data[$data[name]] = sprintf("%04d", $this->form_data["$data[name]_year"]);
	  	  $this->form_data[$data[name]] .= sprintf("%02d", $this->form_data["$data[name]_month"]);
	  	  $this->form_data[$data[name]] .= sprintf("%02d", $this->form_data["$data[name]_day"]);
	  	  $this->form_data[$data[name]] .= sprintf("%02d", $this->form_data["$data[name]_hour"]);
	  	  $this->form_data[$data[name]] .= sprintf("%02d", $this->form_data["$data[name]_minute"]);
	  	  $this->form_data[$data[name]] .= sprintf("%02d", $this->form_data["$data[name]_second"]);
	  	  unset($this->form_data["$data[name]_year"]);
	  	  unset($this->form_data["$data[name]_month"]);
	  	  unset($this->form_data["$data[name]_day"]);
	  	  unset($this->form_data["$data[name]_hour"]);
	  	  unset($this->form_data["$data[name]_minute"]);
	  	  unset($this->form_data["$data[name]_second"]);
		 	} else {
				$error .= "$data[title] is not a valid date.\n";
		 	}
		}
  	$result->MoveNext();
 	}

	  if (isset($this->form_data[password]) && $this->form_data[password] != '') {
		  if ($this->form_data[password] != $this->form_data[password_verify]) {
			  $error .= "Password verification did not match\n";
		  } else {
			  unset($this->form_data[password_verify]);
		  }
	  } else {
		  unset($this->form_data[password]);
		  unset($this->form_data[password_verify]);
	  }

	  $error = nl2br($error);
	  $gui->interface_data[error] = $error;
 	  method_exists($this, 'validateFormLocal') ? $this->validateFormLocal() : '';
	  $gui->theme->assign('idata', $gui->interface_data);
	  if ($error == '' ) { return 1; } else { return 0; }
  }
}
?>